/**
 * MAC440  - Sistemas de Objectos Distribudos 
 * Aluno   - Charlie Yukio Nakagawa - 2236341
 *         - Gustavo Castilhos      - 1142713
 * Projeto - Autenticacao com Interceptadores no JDK 1.4
 **/
package autentica.module;

import java.util.*;
import java.io.*;
import javax.security.auth.*;
import javax.security.auth.callback.*;
import javax.security.auth.login.*;
import javax.security.auth.spi.*;
import autentica.principal.AutenticaPrincipal;

/**
 * Classe que implementa a interface LoginModule
 **/
public class AutenticaLoginModule implements LoginModule {
  private Subject subject;
  private CallbackHandler callbackHandler;
  private Map sharedState;
  private Map options;
  private boolean debug = false;
  // estados da autenticacao
  private boolean succeeded = false;
  private boolean commitSucceeded = false;
  // username and password
  private String username;
  private char[] password;
  private AutenticaPrincipal userPrincipal;
  /**
   * Inicializa
   **/
  public void initialize(Subject subject, CallbackHandler callbackHandler,
    Map sharedState, Map options) {
 
    this.subject = subject;
    this.callbackHandler = callbackHandler;
    this.sharedState = sharedState;
    this.options = options;
    debug = "true".equalsIgnoreCase((String)options.get("debug"));
  }
  /**
   * Faz o login
   **/
  public boolean login() throws LoginException {
    if (callbackHandler == null)
      throw new LoginException("Error: no CallbackHandler available to garner authentication information from the user");
    Callback[] callbacks = new Callback[2];
    callbacks[0] = new NameCallback("user name: ");
    callbacks[1] = new PasswordCallback("password: ", false);
    try {
      callbackHandler.handle(callbacks);
      username = ((NameCallback)callbacks[0]).getName();
      char[] tmpPassword = ((PasswordCallback)callbacks[1]).getPassword();
      if (tmpPassword == null) {
        tmpPassword = new char[0];
      }
      password = new char[tmpPassword.length];
      System.arraycopy(tmpPassword, 0,
      password, 0, tmpPassword.length);
      ((PasswordCallback)callbacks[1]).clearPassword(); 
    } 
    catch (java.io.IOException ioe) {
      throw new LoginException(ioe.toString());
    } 
    catch (UnsupportedCallbackException uce) {
      throw new LoginException("Error: " + uce.getCallback().toString() + " not available to garner authentication information from the user");
    }
    if (debug) {
      System.out.println("\t\t[SampleLoginModule] user entered user name: " + username);
      System.out.print("\t\t[SampleLoginModule] user entered password: ");
      for (int i = 0; i < password.length; i++)
        System.out.print(password[i]);
      System.out.println();
    }
    // Verifica usuario e senha
    boolean usernameCorrect = false;
    boolean passwordCorrect = false;
    String[] usrpwd = new String[2];
    try {
      FileReader fr = new FileReader("server_pwd.conf");
      StreamTokenizer st = new StreamTokenizer(fr);
      st.commentChar(35);
      int fim = 0;
      while (fim != st.TT_EOF && usrpwd[0] == null) {
        if (st.sval != null) {
          usrpwd[0] = st.sval;
          fim = st.nextToken();       
          usrpwd[1] = st.sval;
        }
        fim = st.nextToken();
      }
    } 
    catch (IOException io) {
      System.out.println(io.getMessage());
    }
    if (username.equals(usrpwd[0]))
      usernameCorrect = true;
    System.out.println(usrpwd[0]);
    System.out.println(usrpwd[1]);
    String pwd = new String(password);
    if (usernameCorrect && pwd.equals(usrpwd[1])) {
      // autenticacao Ok
      passwordCorrect = true;
      if (debug)
      System.out.println("\t\t[SampleLoginModule] authentication succeeded");
        succeeded = true;
      return true;
    } 
    else {
      // autenticacao falhou
      if (debug)
        System.out.println("\t\t[SampleLoginModule] authentication failed");
      succeeded = false;
      username = null;
      for (int i = 0; i < password.length; i++)
        password[i] = ' ';
      password = null;
      if (!usernameCorrect) {
        throw new FailedLoginException("User Name Incorrect");
      }
      else {
        throw new FailedLoginException("Password Incorrect");
      }
    }
  }
  /**
   * Efetua o login
   **/
  public boolean commit() throws LoginException {
    if (succeeded == false) {
      return false;
    } 
    else {
      userPrincipal = new AutenticaPrincipal(username);
      if (!subject.getPrincipals().contains(userPrincipal))
        subject.getPrincipals().add(userPrincipal);
      if (debug) {
        System.out.println("\t\t[SampleLoginModule] added SamplePrincipal to Subject");
      }
      // in any case, clean out state
      username = null;
      for (int i = 0; i < password.length; i++)
        password[i] = ' ';
      password = null;
      commitSucceeded = true;
      return true;
    }
  }
  /**
   * Aborta um login
   **/
  public boolean abort() throws LoginException {
    if (succeeded == false) {
      return false;
    } 
    else 
      if (succeeded == true && commitSucceeded == false) {
        // login succeeded but overall authentication failed
        succeeded = false;
        username = null;
        if (password != null) {
          for (int i = 0; i < password.length; i++)
            password[i] = ' ';
          password = null;
        }
        userPrincipal = null;
      } 
      else {
        // overall authentication succeeded and commit succeeded,
        // but someone else's commit failed
        logout();
      }
      return true;
  }
  /**
   * Faz o logout de um cliente
   **/
  public boolean logout() throws LoginException {
    subject.getPrincipals().remove(userPrincipal);
    succeeded = false;
    succeeded = commitSucceeded;
    username = null;
    if (password != null) {
      for (int i = 0; i < password.length; i++)
        password[i] = ' ';
      password = null;
    }
    userPrincipal = null;
    return true;
  }
}
