/**
 * MAC440  - Sistemas de Objectos Distribudos 
 * Aluno   - Charlie Yukio Nakagawa - 2236341
 *         - Gustavo Castilhos      - 1142713
 * Projeto - Autenticacao com Interceptadores no JDK 1.4
 **/
package interceptor;
import org.omg.PortableInterceptor.*;
import org.omg.CORBA.Any;
import java.util.*;
import java.io.*;
import org.omg.IOP.*;
import java.io.StreamTokenizer;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
	
/**
 * ClientLogInterceptor 
 * Classe que faz as interceptacoes do lado do cliente
 **/
public class ClientLogInterceptor extends org.omg.CORBA.LocalObject
                                  implements ClientRequestInterceptor {
  private Calendar date;
  private Hashtable tableTable;
  private PrintStream logStream;
  private String [] usrpwd;
  
  /** 
   * constructor 
   **/
  public ClientLogInterceptor() {
    date = Calendar.getInstance();
    tableTable = new Hashtable();
    logStream = System.out;
    usrpwd = new String[2];
    try {
      FileReader fr = new FileReader("client_pwd.conf");
      StreamTokenizer st = new StreamTokenizer(fr);
      st.commentChar(35);
      int fim = 0;
      String user = "", passwd = "";
      while (fim != st.TT_EOF && usrpwd[0] == null) {
        if (st.sval != null) {
          usrpwd[0] = st.sval;
          fim = st.nextToken();       
          usrpwd[1] = st.sval;
        }
        fim = st.nextToken();
      }
    }
    catch (IOException io) {
      logStream.println(io.getMessage());
    }
  }
  /**
   * Delvolve o nome
   **/
  public String name() {
    return "ClientLogInterceptor";
  }
  /**
   * Envio de requisicao ao servidor
   **/
  public void send_request(ClientRequestInfo ri) throws ForwardRequest {
    try {
      Calendar cal = new GregorianCalendar();
      Date data = cal.getTime();
      DateFormat dataFormatada = new SimpleDateFormat("dd 'de' MMMMM 'de' yyyy, hh:mm:ss");
      log("** Log time: " + dataFormatada.format(data));
      log("Id: " + ri.request_id() + "\nsend_request");
    }
    catch (IOException io) {
    }
    if (ri.response_expected()) {
      Integer id = new Integer(ri.request_id());
      Hashtable table = (Hashtable)tableTable.get(ri.target());
      if (table == null) {
        table = new Hashtable();
        tableTable.put(ri.target(), table);
      }
      table.put(id, new Long(System.currentTimeMillis()));
    }
    try {
      byte [] bt = (usrpwd[0] + " " + usrpwd[1]).getBytes();
      ri.add_request_service_context(new ServiceContext(77, bt) , true);
    }
    catch (Exception e) {
      logStream.println("** " + e.getMessage());
    }
  }
  /**
   * Resposta da requisicao
   **/
  public void receive_reply(ClientRequestInfo ri) {
    String str = "";
    try {
      str = new String((ri.get_reply_service_context(79)).context_data);
    }
    catch (Exception e) {
      //keep going
    }
    try {  
      log("receive_reply");
      if (!str.equals("")) {
        log(str);
      }
      Hashtable table = (Hashtable) tableTable.get(ri.target());
      if (table == null) {
        logStream.println("error: no request table for object");
        return;
      }
      
      long t = System.currentTimeMillis();
      Long startTime = (Long) table.remove(new Integer(ri.request_id()));
      
      if (startTime != null) {
        log("id: " + ri.request_id() +
            " operation: " + ri.operation() +
            " roundtrip: " + 
            (t - startTime.longValue()) +
            "msecs. ");
      }                  
      else 
        log("** Resposta inesperada para requisio " +
            ri.request_id());
    }
    catch (IOException io) {
    }
  }
  
  //empty methods
  public void send_poll(ClientRequestInfo ri) {
    try {
      log("send_poll");
    }
    catch (IOException io) {
    }
  }
  /**
   * Recebe excecao do servidor
   **/
  public void receive_exception(ClientRequestInfo ri) throws ForwardRequest {
    String str = "";
    String excecao = "";
    try {
      try {
        //Desempacota o nome da exceo
        excecao = ((ri.received_exception()).type()).name();
      }
      catch (org.omg.CORBA.TypeCodePackage.BadKind e) {
        log("** Erro no desempacotamento do ORB: " + e.getMessage());
      }
      if (excecao.equals("org.omg.CORBA.UNKNOWN")) {
        str = new String((ri.get_reply_service_context(78)).context_data);
        log("receive_exception: " + ri.received_exception_id() + "\n" + str);
      }
    }
    catch (IOException io) {
      System.out.println("Erro no arquivo de log: " + io.getMessage());
    }
  }
  /**
   * Recebe outras informacoes do interceptador do servidor
   **/
  public void receive_other(ClientRequestInfo ri) throws ForwardRequest {
    try {
      log("receive_other");
    }
    catch (IOException io) {
    }
  }
  
  public void destroy() {
  }
  /**
   * Armazena informacoes de log
   **/
  private void log(String logstr) throws IOException {
    File f = new File("client_interceptor.log");
    FileOutputStream fos = new FileOutputStream(f, true);
    fos.write((logstr + "\n").getBytes());
    fos.close();
  }
}